@@ -1,9 +1,13 @@
 #include "displayserver_linux.h"
 
-#ifdef FF_HAVE_X11
+#ifdef FF_HAVE_XRANDR
+
 #include "common/library.h"
 #include "common/parsing.h"
+#include "util/edidHelper.h"
 #include "util/stringUtils.h"
+
+#include <X11/extensions/Xrandr.h>
 #include <X11/Xlib.h>
 
 typedef struct X11PropertyData
@@ -58,62 +62,6 @@ static void x11DetectWMFromEWMH(X11PropertyData* data, Display* display, FFDispl
     data->ffXFree(wmWindow);
 }
 
-void ffdsConnectXlib(FFDisplayServerResult* result)
-{
-    FF_LIBRARY_LOAD(x11, &instance.config.library.libX11, , "libX11" FF_LIBRARY_EXTENSION, 7, "libX11-xcb" FF_LIBRARY_EXTENSION, 2)
-    FF_LIBRARY_LOAD_SYMBOL(x11, XOpenDisplay,)
-    FF_LIBRARY_LOAD_SYMBOL(x11, XCloseDisplay,)
-
-    X11PropertyData propertyData;
-    bool propertyDataInitialized = x11InitPropertyData(x11, &propertyData);
-
-    Display* display = ffXOpenDisplay(x11);
-    if(display == NULL)
-        return;
-
-    if(propertyDataInitialized && ScreenCount(display) > 0)
-        x11DetectWMFromEWMH(&propertyData, display, result);
-
-    for(int i = 0; i < ScreenCount(display); i++)
-    {
-        Screen* screen = ScreenOfDisplay(display, i);
-        ffdsAppendDisplay(result,
-            (uint32_t) WidthOfScreen(screen),
-            (uint32_t) HeightOfScreen(screen),
-            0,
-            (uint32_t) WidthOfScreen(screen),
-            (uint32_t) HeightOfScreen(screen),
-            0,
-            NULL,
-            FF_DISPLAY_TYPE_UNKNOWN,
-            false,
-            0,
-            (uint32_t) WidthMMOfScreen(screen),
-            (uint32_t) HeightMMOfScreen(screen)
-        );
-    }
-
-    ffXCloseDisplay(display);
-
-    //If wayland hasn't set this, connection failed for it. So we are running only a X Server, not XWayland.
-    if(result->wmProtocolName.length == 0)
-        ffStrbufSetS(&result->wmProtocolName, FF_WM_PROTOCOL_X11);
-}
-
-#else
-
-void ffdsConnectXlib(FFDisplayServerResult* result)
-{
-    //Do nothing. WM / DE detection will use environment vars to detect as much as possible.
-    FF_UNUSED(result);
-}
-
-#endif //FF_HAVE_X11
-
-#ifdef FF_HAVE_XRANDR
-#include "util/edidHelper.h"
-#include <X11/extensions/Xrandr.h>
-
 typedef struct XrandrData
 {
     FF_LIBRARY_SYMBOL(XInternAtom)
@@ -137,26 +85,13 @@ typedef struct XrandrData
     XRRScreenResources* screenResources;
 } XrandrData;
 
-static double xrandrHandleMode(XrandrData* data, RRMode mode)
-{
-    for(int i = 0; i < data->screenResources->nmode; i++)
-    {
-        if(data->screenResources->modes[i].id == mode)
-        {
-            XRRModeInfo* modeInfo = &data->screenResources->modes[i];
-            return (double) modeInfo->dotClock / (double) (modeInfo->hTotal * modeInfo->vTotal);
-        }
-    }
-    return 0;
-}
-
-static bool xrandrHandleCrtc(XrandrData* data, RRCrtc crtc, FFstrbuf* name, bool primary, XRROutputInfo* output, FFDisplayType displayType)
+static bool xrandrHandleCrtc(XrandrData* data, XRROutputInfo* output, FFstrbuf* name, bool primary, FFDisplayType displayType, uint8_t* edidData, uint32_t edidLength)
 {
     //We do the check here, because we want the best fallback display if this call failed
     if(data->screenResources == NULL)
         return false;
 
-    XRRCrtcInfo* crtcInfo = data->ffXRRGetCrtcInfo(data->display, data->screenResources, crtc);
+    XRRCrtcInfo* crtcInfo = data->ffXRRGetCrtcInfo(data->display, data->screenResources, output->crtc);
     if(crtcInfo == NULL)
         return false;
 
@@ -177,24 +112,50 @@ static bool xrandrHandleCrtc(XrandrData* data, RRCrtc crtc, FFstrbuf* name, bool
             break;
     }
 
-    bool res = ffdsAppendDisplay(
+    XRRModeInfo* currentMode = NULL;
+    if (data->screenResources)
+    {
+        for(int i = 0; i < data->screenResources->nmode; i++)
+        {
+            if(data->screenResources->modes[i].id == crtcInfo->mode)
+            {
+                currentMode = &data->screenResources->modes[i];
+                break;
+            }
+        }
+    }
+
+    XRRModeInfo* preferredMode = data->screenResources && output->npreferred > 0 ? &data->screenResources->modes[0] : NULL;
+
+    FFDisplayResult* item = ffdsAppendDisplay(
         data->result,
         (uint32_t) crtcInfo->width,
         (uint32_t) crtcInfo->height,
-        xrandrHandleMode(data, crtcInfo->mode),
+        currentMode ? (double) currentMode->dotClock / (double) ((uint32_t) currentMode->hTotal * currentMode->vTotal) : 0,
         (uint32_t) crtcInfo->width,
         (uint32_t) crtcInfo->height,
+        preferredMode ? (uint32_t) preferredMode->width : 0,
+        preferredMode ? (uint32_t) preferredMode->height : 0,
+        preferredMode ? (double) preferredMode->dotClock / (double) ((uint32_t) preferredMode->hTotal * preferredMode->vTotal) : 0,
         rotation,
         name,
         displayType,
         primary,
         0,
         (uint32_t) output->mm_width,
-        (uint32_t) output->mm_height
+        (uint32_t) output->mm_height,
+        "xlib-randr-crtc"
     );
 
+    if (edidLength)
+    {
+        item->hdrStatus = ffEdidGetHdrCompatible(edidData, edidLength) ? FF_DISPLAY_HDR_STATUS_SUPPORTED : FF_DISPLAY_HDR_STATUS_UNSUPPORTED;
+        ffEdidGetSerialAndManufactureDate(edidData, &item->serial, &item->manufactureYear, &item->manufactureWeek);
+        ffEdidGetPhysicalSize(edidData, &item->physicalWidth, &item->physicalHeight);
+    }
+
     data->ffXRRFreeCrtcInfo(crtcInfo);
-    return res;
+    return !!item;
 }
 
 static bool xrandrHandleOutput(XrandrData* data, RROutput output, FFstrbuf* name, bool primary, FFDisplayType displayType)
@@ -203,26 +164,30 @@ static bool xrandrHandleOutput(XrandrData* data, RROutput output, FFstrbuf* name
     if(outputInfo == NULL)
         return false;
 
+    uint8_t* edidData = NULL;
+    unsigned long edidLength = 0;
     Atom atomEdid = data->ffXInternAtom(data->display, "EDID", true);
     if (atomEdid != None)
     {
         int actual_format = 0;
-        unsigned long nitems = 0, bytes_after = 0;
+        unsigned long bytes_after = 0;
         Atom actual_type = None;
-        uint8_t* edidData = NULL;
-        if (data->ffXRRGetOutputProperty(data->display, output, atomEdid, 0, 100, false, false, AnyPropertyType, &actual_type, &actual_format, &nitems, &bytes_after, &edidData) == Success)
+        if (data->ffXRRGetOutputProperty(data->display, output, atomEdid, 0, 100, false, false, AnyPropertyType, &actual_type, &actual_format, &edidLength, &bytes_after, &edidData) == Success)
         {
-            if (nitems >= 128)
+            if (edidLength >= 128)
             {
                 ffStrbufClear(name);
                 ffEdidGetName(edidData, name);
             }
+            else
+                edidLength = 0;
         }
-        if (edidData)
-            data->ffXFree(edidData);
     }
-    bool res = xrandrHandleCrtc(data, outputInfo->crtc, name, primary, outputInfo, displayType);
 
+    bool res = xrandrHandleCrtc(data, outputInfo, name, primary, displayType, edidData, (uint32_t) edidLength);
+
+    if (edidData)
+        data->ffXFree(edidData);
     data->ffXRRFreeOutputInfo(outputInfo);
 
     return res;
@@ -248,13 +213,15 @@ static bool xrandrHandleMonitor(XrandrData* data, XRRMonitorInfo* monitorInfo)
         0,
         (uint32_t) monitorInfo->width,
         (uint32_t) monitorInfo->height,
+        0, 0, 0,
         0,
         &name,
         displayType,
         !!monitorInfo->primary,
         0,
         (uint32_t) monitorInfo->mwidth,
-        (uint32_t) monitorInfo->mheight
+        (uint32_t) monitorInfo->mheight,
+        "xlib-randr-monitor"
     );
 }
 
@@ -298,44 +265,46 @@ static void xrandrHandleScreen(XrandrData* data, Screen* screen)
         0,
         (uint32_t) WidthOfScreen(screen),
         (uint32_t) HeightOfScreen(screen),
+        0, 0, 0,
         0,
         NULL,
         FF_DISPLAY_TYPE_UNKNOWN,
         false,
         0,
         (uint32_t) WidthMMOfScreen(screen),
-        (uint32_t) HeightMMOfScreen(screen)
+        (uint32_t) HeightMMOfScreen(screen),
+        "xlib_randr_screen"
     );
 }
 
-void ffdsConnectXrandr(FFDisplayServerResult* result)
+const char* ffdsConnectXrandr(FFDisplayServerResult* result)
 {
-    FF_LIBRARY_LOAD(xrandr, &instance.config.library.libXrandr, , "libXrandr" FF_LIBRARY_EXTENSION, 3)
+    FF_LIBRARY_LOAD(xrandr, "dlopen libXrandr failed", "libXrandr" FF_LIBRARY_EXTENSION, 3)
 
-    FF_LIBRARY_LOAD_SYMBOL(xrandr, XOpenDisplay,)
-    FF_LIBRARY_LOAD_SYMBOL(xrandr, XCloseDisplay,)
+    FF_LIBRARY_LOAD_SYMBOL_MESSAGE(xrandr, XOpenDisplay)
+    FF_LIBRARY_LOAD_SYMBOL_MESSAGE(xrandr, XCloseDisplay)
 
     XrandrData data;
 
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XInternAtom,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XGetAtomName,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XFree,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRGetMonitors,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRGetScreenResourcesCurrent,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRGetOutputInfo,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRGetOutputProperty,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRGetCrtcInfo,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRFreeCrtcInfo,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRFreeOutputInfo,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRFreeScreenResources,);
-    FF_LIBRARY_LOAD_SYMBOL_VAR(xrandr, data, XRRFreeMonitors,);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XInternAtom);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XGetAtomName);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XFree);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetMonitors);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetScreenResourcesCurrent);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetOutputInfo);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetOutputProperty);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetCrtcInfo);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRFreeCrtcInfo);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRFreeOutputInfo);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRFreeScreenResources);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRFreeMonitors);
 
     X11PropertyData propertyData;
     bool propertyDataInitialized = x11InitPropertyData(xrandr, &propertyData);
 
     data.display = ffXOpenDisplay(NULL);
     if(data.display == NULL)
-        return;
+        return "XOpenDisplay() failed";
 
     if(propertyDataInitialized && ScreenCount(data.display) > 0)
         x11DetectWMFromEWMH(&propertyData, data.display, result);
@@ -350,14 +319,17 @@ void ffdsConnectXrandr(FFDisplayServerResult* result)
     //If wayland hasn't set this, connection failed for it. So we are running only a X Server, not XWayland.
     if(result->wmProtocolName.length == 0)
         ffStrbufSetS(&result->wmProtocolName, FF_WM_PROTOCOL_X11);
+
+    return NULL;
 }
 
 #else
 
-void ffdsConnectXrandr(FFDisplayServerResult* result)
+const char* ffdsConnectXrandr(FFDisplayServerResult* result)
 {
     //Do nothing here. There are more x11 implementations to come.
     FF_UNUSED(result);
+    return "Fastfetch was compiled without libXrandr support";
 }
 
 #endif // FF_HAVE_XRANDR
