@@ -3,6 +3,7 @@
 #include "common/thread.h"
 #include "detection/gtk_qt/gtk_qt.h"
 #include "detection/displayserver/displayserver.h"
+#include "util/stringUtils.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -17,7 +18,7 @@ static inline bool allValuesSet(const FFQtResult* result)
         result->wallpaper.length > 0;
 }
 
-typedef enum PlasmaCategory
+typedef enum __attribute__((__packed__)) PlasmaCategory
 {
     PLASMA_CATEGORY_GENERAL,
     PLASMA_CATEGORY_KDE,
@@ -43,11 +44,11 @@ static bool detectPlasmaFromFile(const char* filename, FFQtResult* result)
             char categoryName[32];
             sscanf(line, "[%31[^]]", categoryName);
 
-            if(strcasecmp(categoryName, "General") == 0)
+            if(ffStrEqualsIgnCase(categoryName, "General"))
                 category = PLASMA_CATEGORY_GENERAL;
-            else if(strcasecmp(categoryName, "KDE") == 0)
+            else if(ffStrEqualsIgnCase(categoryName, "KDE"))
                 category = PLASMA_CATEGORY_KDE;
-            else if(strcasecmp(categoryName, "Icons") == 0)
+            else if(ffStrEqualsIgnCase(categoryName, "Icons"))
                 category = PLASMA_CATEGORY_ICONS;
             else
                 category = PLASMA_CATEGORY_OTHER;
@@ -134,6 +135,34 @@ static void detectLXQt(FFQtResult* result)
     ffParsePropFileConfig("pcmanfm-qt/lxqt/settings.conf", "Wallpaper=", &result->wallpaper);
 }
 
+static void detectQtCt(char qver, FFQtResult* result)
+{
+    // qt5ct and qt6ct are technically separate applications, but they're both
+    // by the same author and qt6ct understands qt5ct in qt6 applications as well.
+    char file[] = "qtXct/qtXct.conf";
+    file[2] = file[8] = qver;
+    ffParsePropFileConfigValues(file, 3, (FFpropquery[]) {
+        {"style=", &result->widgetStyle},
+        {"icon_theme=", &result->icons},
+        {"general=", &result->font}
+    });
+
+    if (ffStrbufStartsWithC(&result->font, '@'))
+    {
+        // See QVariant notes on https://doc.qt.io/qt-5/qsettings.html and
+        // https://github.com/fastfetch-cli/fastfetch/issues/1053#issuecomment-2197254769
+        // Thankfully, newer versions use the more common font encoding.
+        ffStrbufSetNS(&result->font, 5, file);
+    }
+}
+
+static void detectKvantum(FFQtResult* result)
+{
+    ffParsePropFileConfigValues("Kvantum/kvantum.kvconfig", 1, (FFpropquery[]) {
+        {"theme=", &result->widgetStyle},
+    });
+}
+
 const FFQtResult* ffDetectQt(void)
 {
     static FFQtResult result;
@@ -150,11 +179,20 @@ const FFQtResult* ffDetectQt(void)
     ffStrbufInit(&result.wallpaper);
 
     const FFDisplayServerResult* wmde = ffConnectDisplayServer();
+    const char *qplatformtheme = getenv("QT_QPA_PLATFORMTHEME");
 
-    if(ffStrbufIgnCaseCompS(&wmde->dePrettyName, FF_DE_PRETTY_PLASMA) == 0)
+    if(ffStrbufIgnCaseEqualS(&wmde->dePrettyName, FF_DE_PRETTY_PLASMA))
         detectPlasma(&result);
-    else if(ffStrbufIgnCaseCompS(&wmde->dePrettyName, FF_DE_PRETTY_LXQT) == 0)
+    else if(ffStrbufIgnCaseEqualS(&wmde->dePrettyName, FF_DE_PRETTY_LXQT))
         detectLXQt(&result);
+    else if(ffStrSet(qplatformtheme) && (ffStrEquals(qplatformtheme, "qt5ct") || ffStrEquals(qplatformtheme, "qt6ct")))
+        detectQtCt(qplatformtheme[2], &result);
+
+    if(ffStrbufEqualS(&result.widgetStyle, "kvantum") || ffStrbufEqualS(&result.widgetStyle, "kvantum-dark"))
+    {
+        ffStrbufClear(&result.widgetStyle);
+        detectKvantum(&result);
+    }
 
     return &result;
 }
