@@ -129,6 +129,12 @@ bool ffParseNetIOCommandOptions(FFNetIOOptions* options, const char* key, const
         return true;
     }
 
+    if (ffStrEqualsIgnCase(subKey, "wait-time"))
+    {
+        options->waitTime = ffOptionParseUInt32(key, value);
+        return true;
+    }
+
     return false;
 }
 
@@ -163,6 +169,12 @@ void ffParseNetIOJsonObject(FFNetIOOptions* options, yyjson_val* module)
             continue;
         }
 
+        if (ffStrEqualsIgnCase(key, "waitTime"))
+        {
+            options->waitTime = (uint32_t) yyjson_get_uint(val);
+            continue;
+        }
+
         ffPrintError(FF_NETIO_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
     }
 }
@@ -252,13 +264,14 @@ void ffInitNetIOOptions(FFNetIOOptions* options)
 
     ffStrbufInit(&options->namePrefix);
     options->defaultRouteOnly =
-        #ifdef __ANDROID__
+        #if __ANDROID__ || __OpenBSD__
             false
         #else
             true
         #endif
     ;
     options->detectTotal = false;
+    options->waitTime = 1000;
 }
 
 void ffDestroyNetIOOptions(FFNetIOOptions* options)
